unit CustomerDetails;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  PropertySheet, StdCtrls, ExtCtrls, ComCtrls;

type
  TfrmCustomerDetails = class(TfrmPropertySheet)
    tabOrders: TTabSheet;
    lblName: TLabel;
    edtName: TEdit;
    mmoAddress: TMemo;
    lblAddress: TLabel;
    edtPostcode: TEdit;
    lblPostcode: TLabel;
    edtTelephoneNumber: TEdit;
    lblTelephoneNumber: TLabel;
    lblOrders: TLabel;
    Image1: TImage;
    lvwOrders: TListView;
    btnDelete: TButton;
    btnEdit: TButton;
    btnNew: TButton;
    chkLatePayer: TCheckBox;
    chkDiscounted: TCheckBox;
    chkNoDeliveryCharge: TCheckBox;
    procedure lvwOrdersChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure btnNewClick(Sender: TObject);
  private
  protected
    procedure LoadTab (TabSheet: TTabSheet; ObjectToLoad: TObject); override;
    procedure SaveTab (TabSheet: TTabSheet; ObjectToSave: TObject); override;
  public
  end;

implementation

uses
  ProblemDomain, OrderDetails;

{$R *.DFM}

procedure TfrmCustomerDetails.LoadTab (TabSheet: TTabSheet; ObjectToLoad: TObject);
begin
  with TCustomer (ObjectToLoad) do begin
    if TabSheet = tabDetails then begin
      edtName.Text := Name;
      mmoAddress.Lines.Assign (Address);
      edtPostcode.Text := Postcode;
      edtTelephoneNumber.Text := TelephoneNumber;
      chkLatePayer.Checked := (caLatePayer in Attributes);
      chkDiscounted.Checked := (caDiscounted in Attributes);
      chkNoDeliveryCharge.Checked := (caNoDeliveryCharge in Attributes);
    end else if TabSheet = tabOrders then begin
      // Nothing to do in this demo but in the real world this would populate the list view
      // with related data.
      //
      // In a "pure" OO world you would pass the set of business objects involved,
      // in this case the orders for a customer, which would look something like this:
      // LoadList (Customer.Orders);
      //
      // In the "traditional" world you could create some kind of recordset and run
      // through it, maybe a TQuery or TDataset descendant. You could do it all by hand
      // or use datamodules, or create generic parameterised routines:
      // LoadList (TQuery.Create ('SELECT * FROM Orders WHERE CustomerID=' + IntToStr (Customer.ID)));
      //
      // Alternatively, by centralising the code in a datamodule:
      //
      // DataModule.qryOrdersForCustomers.ParamByName ('ID').AsInteger := Customer.ID;
      // LoadList (DataModule.qryOrdersForCustomers);
      //
      // The opportunities are endless, but the essence is to perform database access and populate
      // the screen in one go. Note that this always involves stateless access to the queries and
      // so the same query can safely be re-used throughout the application. Note also that these
      // queries mimic how databases "like" to be used: unidirectional read-only queries.
    end;
  end;
end;

procedure TfrmCustomerDetails.SaveTab (TabSheet: TTabSheet; ObjectToSave: TObject);
begin
  with TCustomer (ObjectToSave) do begin
    if TabSheet = tabDetails then begin
      Name := edtName.Text;
      Address.Assign (mmoAddress.Lines);
      Postcode := edtPostcode.Text;
      TelephoneNumber := edtTelephoneNumber.Text;
      Attributes := [];
      if chkLatePayer.Checked then Attributes := Attributes + [caLatePayer];
      if chkDiscounted.Checked then Attributes := Attributes + [caDiscounted];
      if chkNoDeliveryCharge.Checked then Attributes := Attributes + [caNoDeliveryCharge];
    end else if TabSheet = tabOrders then begin
      // Nothing to do
    end;
  end;
end;

// Events

procedure TfrmCustomerDetails.lvwOrdersChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
  inherited;
  btnEdit.Enabled := (Item <> nil);
  btnDelete.Enabled := (Item <> nil);
end;

procedure TfrmCustomerDetails.btnNewClick(Sender: TObject);
var
  Order: TOrder;
begin
  inherited;
  Order := TOrder.Create;
  try
    Order.Customer.Name := edtName.Text;
    TfrmOrderDetails.Edit (Order);
  finally
    Order.Free;
  end;
end;

end.
